/********************************** <lalVerbatim file="ExtTrigDataCV">
Author: Rahkola, R.
$Id: ExtTrigData.c,v 1.2 2002/04/18 19:38:14 rrahkola Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="ExtTrigDataC">
\subsection{Module \texttt{ExtTrigData.c}}
\label{s:ExtTrigData.c}

Routines for creating and destroying a 
\texttt{ETDataSegmentVector}.

\subsubsection*{Prototypes}
\input{ExtTrigDataCP}
\index{\texttt{LALCreateETDataSegmentVector()}}

\subsection*{Description}

Creates and destroys a \texttt{ETDataSegmentVector},
which holds the input time-series data for two interferometers and a data
quality channel.

\subsection*{Algorithms}
  
\subsection*{Uses}

\begin{itemize}
\item \texttt{LALMalloc}
\item \texttt{LALFree}
\item \texttt{LALCreateVector}
\item \texttt{LALDestroyVector}
\end{itemize}

\subsection*{Notes}

\vfill{\footnotesize\input{ExtTrigDataCV}}

*************************************************** </lalLaTeX> */

#include <lal/LALStdlib.h>
#include <lal/SeqFactories.h>
#include "ExtTrig.h"

NRCSID (EXTTRIGDATAC, "$Id: ExtTrigData.c,v 1.2 2002/04/18 19:38:14 rrahkola Exp $");

/************************ <lalVerbatim file="ExtTrigDataCP"> */
void LALCreateETDataSegmentVector (
				   LALStatus            *status,
				   ETDataSegmentVector **vector,
				   ETInitParams         *params
)
/*********************************************** </lalVerbatim> */
{
    
  INT4                    i;              /* iterator for 'for' loops        */
  UINT4                   numPoints;      /* number of points in timeSeries  */
  UINT4                   filtLength;     /* number of points in freqSeries  */
  ETDataSegmentVector    *vectorPtr;      /* pointer to 'vector' arg         */
  ETDataSegment          *segPtr;         /* pointer to index in 'vector'    */
    
  INITSTATUS (status, "LALCreateETDataSegmentVector", EXTTRIGDATAC);
  ATTATCHSTATUSPTR (status);

  /* 
   *
   * check output vector and parameters 
   *
   */
  
  ASSERT ( !*vector, status, EXTTRIGDATAH_ENNUL, EXTTRIGDATAH_MSGENNUL );
  ASSERT (   params, status, EXTTRIGDATAH_ENULL, EXTTRIGDATAH_MSGENULL );
/*  if ( params->numSegments <= 0 ) {
      ABORT ( status, EXTTRIGDATAH_ENSEG, EXTTRIGDATAH_MSGENSEG );
    }
  if (params->numPoints <= 0) { 
      ABORT ( status, EXTTRIGDATAH_ENPTS, EXTTRIGDATAH_MSGENPTS );
    }
  
  if (params->numFrequencies <= 0) {
      ABORT (status, 
	     EXTTRIGDATAH_ENFRQ, 
	     EXTTRIGDATAH_MSGENFRQ);
    }
*/  
  /* vector length */
  numPoints = params->segLength;
  filtLength = params->segLength;

  /*
   *
   * allocate memory 
   *
   */

  /* pointer to vector */
  vectorPtr = *vector = (ETDataSegmentVector *) 
    LALMalloc (sizeof (ETDataSegmentVector));
  if ( !vector ) {
      ABORT (status, EXTTRIGDATAH_EALOC, EXTTRIGDATAH_MSGEALOC );
    }
  
  /* assign vector length, segsToBeAnalyzed, and segsAnalyzed */ 
  /* :TODO: include 'off'-source segments as well as the single 'on'-source */
  vectorPtr->length = params->numOffSegments + 1;
  //vectorPtr->length = 1;
  vectorPtr->segsToBeAnalyzed = vectorPtr->length;
  vectorPtr->segsAnalyzed     = 0;
#if DEBUG_EXTTRIG > 2
  fprintf( stdout, "Creating ETDataSegmentVector,\tlength %d \n",
	   vectorPtr->length );
  fflush( stdout );
#endif // DEBUG_EXTTRIG > 2
  
  
  /* pointer to segment */
  /* :TODO: include 'off'-source segments as well as the single 'on'-source */
  segPtr = vectorPtr->data = (ETDataSegment *) 
    LALMalloc (vectorPtr->length * sizeof (ETDataSegment));
  if (!segPtr)
    {
      /* :TODO: is LALFree() really needed here?? */
      LALFree( vectorPtr );
      vectorPtr = NULL;
      ABORT( status, EXTTRIGDATAH_EALOC, EXTTRIGDATAH_MSGEALOC );
    }
  
  /* individual data segments */ 
  for (i = 0; i < vectorPtr->length ; ++i)
    {
      /* tsChan1 */
      segPtr[i].tsChan1 = (REAL4TimeSeries *) 
	LALMalloc (sizeof (REAL4TimeSeries));
      if (!(segPtr[i].tsChan1))
        {
	  ABORT (status, EXTTRIGDATAH_EALOC, EXTTRIGDATAH_MSGEALOC);
        }
      
      segPtr[i].tsChan1->data = NULL;
      LALCreateVector (status->statusPtr, &(segPtr[i].tsChan1->data), 
		       numPoints);
      CHECKSTATUSPTR (status);
      
      /* tsChan2 */
      segPtr[i].tsChan2 = (REAL4TimeSeries *) 
	LALMalloc (sizeof (REAL4TimeSeries));
      if (!(segPtr[i].tsChan2))
        {
	  ABORT (status, EXTTRIGDATAH_EALOC, EXTTRIGDATAH_MSGEALOC);
        }
      
      segPtr[i].tsChan2->data = NULL;
      LALCreateVector (status->statusPtr, &(segPtr[i].tsChan2->data), 
		       numPoints);
      CHECKSTATUSPTR (status);

      /* filter */
      segPtr[i].filter = (COMPLEX8FrequencySeries *) 
	LALCalloc ( 1, sizeof (COMPLEX8FrequencySeries) );
      if (!(segPtr[i].tsChan1))
        {
	  ABORT (status, EXTTRIGDATAH_EALOC, EXTTRIGDATAH_MSGEALOC);
        }
      
      segPtr[i].filter->data = NULL;
      LALCCreateVector (status->statusPtr, &(segPtr[i].filter->data), 
			filtLength );
      CHECKSTATUSPTR (status);

    }
  
  /* normal exit */
  DETATCHSTATUSPTR (status);
  RETURN (status);
}   


/************************ <lalVerbatim file="ExtTrigDataCP"> */
void LALDestroyETDataSegmentVector(
				   LALStatus*              status,
				   ETDataSegmentVector**   vector
)
/*********************************************** </lalVerbatim> */
{
  
  INT4                    i;
  ETDataSegment          *segPtr;
  
  
  INITSTATUS (status, "LALDestroyETDataSegmentVector", EXTTRIGDATAC);
  ATTATCHSTATUSPTR (status);
  
  
  /* 
   *
   * check that there is something that needs to be destroyed 
   *
   */
  ASSERT (*vector, status, EXTTRIGDATAH_ENULL, EXTTRIGDATAH_MSGENULL);
  segPtr = (*vector)->data;
  
  
  /*
   *
   * free memory
   *
   */
#if DEBUG_EXTTRIG > 2
  fprintf( stdout, "Destroying ETDataSegmentVector \n" );
  fflush( stdout );
#endif // DEBUG_EXTTRIG > 2
  /* :TODO: should all pointers be set to NULL after LALFree?? */
  for (i = 0; i < (*vector)->length; ++i)
    {
      LALCDestroyVector( status->statusPtr, &(segPtr[i].filter->data) );
      CHECKSTATUSPTR( status );
      LALFree( segPtr[i].filter );

      LALDestroyVector( status->statusPtr, &(segPtr[i].tsChan2->data) );
      CHECKSTATUSPTR( status );
      LALFree( segPtr[i].tsChan2 );
      
      LALDestroyVector( status->statusPtr, &(segPtr[i].tsChan1->data) );
      CHECKSTATUSPTR( status );
      LALFree( segPtr[i].tsChan1 );
    }
  
  LALFree( segPtr );
  LALFree( *vector );
  *vector = NULL;
  
  /* normal exit */
  DETATCHSTATUSPTR (status);
  RETURN (status);
}


/************************ <lalVerbatim file="ExtTrigDataCP"> */
void LALCreateETSegList (
			 LALStatus     *status,
			 ETSegList    **list,
			 ETInitParams  *params )
/*********************************************** </lalVerbatim> */
{
  INT4                    i;          /* iterator for 'for' loops        */
  UINT4                   window;     /* length of the window (in sec)   */
  UINT4                   listLength; /* number of points in list        */
  ETSegList              *listPtr;    /* pointer to 'list' arg           */
  ETSegment              *segPtr;     /* pointer to index in 'list'      */
    
  INITSTATUS (status, "LALCreateETSegList", EXTTRIGDATAC);
  ATTATCHSTATUSPTR (status);

  /* 
   * check output vector and parameters 
   */
  
  ASSERT ( !*list, status, EXTTRIGDATAH_ENNUL, EXTTRIGDATAH_MSGENNUL );
  ASSERT (   params, status, EXTTRIGDATAH_ENULL, EXTTRIGDATAH_MSGENULL );
  if ( params->numOffSegments <= 0 ) {
    ABORT ( status, EXTTRIGDATAH_ENSEG, EXTTRIGDATAH_MSGENSEG );
  }

  /* vector length = (time available for segments*/
  window = (params->segLength / GW_SAMPLE) + 1;
  listLength = (UINT4) 
    ( (REAL4)(params->offTimeShiftHigh - params->offTimeShiftLow) / 
      (REAL4)window                                                ) + 1;
#if DEBUG_EXTTRIG > 1
#if DEBUG_EXTTRIG > 2
  fprintf( stdout, "Creating ETSegList,\tlength %d \n", listLength );
#endif // DEBUG_EXTTRIG > 2
  fprintf( stdout, 
	   "Number of possible segments: [(%d - %d) / (%d + 1)] + 1 = %d \n",
	   params->offTimeShiftHigh, params->offTimeShiftLow, 
	   (window - 1), listLength                          );
  fflush( stdout );
#endif // DEBUG_EXTTRIG > 1

  /*
   * allocate memory 
   */

  /* pointer to vector */
  listPtr = *list = (ETSegList *) LALMalloc( sizeof(ETSegList) );
  if ( !list ) {
      ABORT (status, EXTTRIGDATAH_EALOC, EXTTRIGDATAH_MSGEALOC );
    }
  
  /* assign list length and filled-segments */ 
  listPtr->length = listLength;
  listPtr->filled = 0;
  
  /* pointer to segment */
  segPtr = listPtr->data = (ETSegment *) 
    LALCalloc( listPtr->length, sizeof (ETSegment) );
  if (!segPtr)
    {
      /* :NOTE: is LALFree() really needed here?? */
      LALFree( listPtr );
      listPtr = NULL;
      ABORT( status, EXTTRIGDATAH_EALOC, EXTTRIGDATAH_MSGEALOC );
    }
  
  /* normal exit */
  DETATCHSTATUSPTR (status);
  RETURN (status);
}   


/************************ <lalVerbatim file="ExtTrigDataCP"> */
void LALDestroyETSegList(
			 LALStatus  *status,
			 ETSegList **list   )
/*********************************************** </lalVerbatim> */
{
  ETSegment    *segPtr;

  INITSTATUS( status, "LALDestroyETSegList", EXTTRIGDATAC );
  ATTATCHSTATUSPTR( status );
  
  /* 
   * check that there is something that needs to be destroyed 
   */
  ASSERT( *list, status, EXTTRIGDATAH_ENULL, EXTTRIGDATAH_MSGENULL );
  segPtr = (*list)->data;

#if DEBUG_EXTTRIG > 2
  fprintf( stdout, "Destroying ETSegList \n" );
  fflush( stdout );
#endif // DEBUG_EXTTRIG > 2

  LALFree( segPtr );
  LALFree( *list );
  *list = NULL;
  
  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}
