/********************** <lalVerbatim file="StochasticBuildOutputCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticBuildOutput.c,v 1.31 2002/04/23 17:04:10 whelan Exp $
 **************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticBuildOutputC">
\subsection{Module \texttt{StochasticBuildOutput.c}}
\label{s:StochasticBuildOutput.c}

Routine that builds the output of the optimally-filtered
cross-correlation statistic spectrum for insertion into the metadata 
database and frames.

\subsubsection*{Prototypes}
\input{StochasticBuildOutputCP}
\index{\texttt{StochasticBuildOutput()}}

\subsection*{Description}

Builds the output of the optimally-filtered cross-correlation
statistic spectrum for subsequent insertion into the metadata
database (as a \texttt{summ\_spectrum} table) and frames (as a
sequence in \texttt{filter\_output}).

\subsection*{Algorithms}

\subsection*{Uses}
 
\begin{itemize}
\item \texttt{LALMalloc}
\item \texttt{LALCalloc}
\end{itemize}

\subsection*{Notes}

\vfill{\footnotesize\input{StochasticBuildOutputCV}}

*************************************************** </lalLaTeX> */



#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/Units.h>
#include "StochasticApplySearch.h"

NRCSID (STOCHASTICBUILDOUTPUTC, "power $Id: StochasticBuildOutput.c,v 1.31 2002/04/23 17:04:10 whelan Exp $");

/************************ <lalVerbatim file="StochasticBuildOutputCP"> */
void StochasticBuildOutput (
      LALStatus*                  status,
      LALSearchOutput*            output,
      StochasticSearchOutput*     input, 
      StochasticSearchParams     *params
)
/*********************************************** </lalVerbatim> */
{
   outPut         *outputPtr;
   dataBase       *dbEntry;
   dataBase       *prev;
   UINT8           segLengthNanoSeconds;
   UINT8           segStartNanoSeconds;
   UINT8           segStopNanoSeconds;
   INT4            stopSec;
   INT4            stopNan;
   UINT4           stringLength;
   UINT4           i;
   multiDimData*   mddEntry;
   CHARVector     *unitString;
   CHAR            string[STOCHASTICAPPLYSEARCHH_MAXSTRINGLENGTH];

   INITSTATUS (status, "StochasticBuildOutput", STOCHASTICBUILDOUTPUTC);
   ATTATCHSTATUSPTR (status);

   output->numOutput = 0;

   if ( input->ccStatReal )
   {
     ++output->numOutput;
   } else if ( input->ccStatComplex )
   {
     output->numOutput += 2;
   }
   if ( input->sigma )
   {
     ++output->numOutput;
   }
   if ( input->ccSpec )
   {
     ++output->numOutput;
   }

   if ( (params->currentSegment == 0) 
	&& ( input->ccStatReal || input->sigma || input->ccStatComplex) )
   {
     output->numOutput += 2;
   }

   if ( !(output->numOutput) )
   {
     DETATCHSTATUSPTR (status);
     RETURN (status);
   }

   /* 
    *
    * create memory for the outPut array
    *
    */

   /* using LALCalloc protects us against new members of the structure */
   outputPtr = output->result 
     = (outPut *) LALCalloc (output->numOutput, sizeof (outPut));
   if (!outputPtr)
   {
      ABORT (status, 
	     STOCHASTICAPPLYSEARCHH_EALOC, 
	     STOCHASTICAPPLYSEARCHH_MSGEALOC);
   }

   /*
    *
    * fill the array of output
    *
    */

   if ( (params->currentSegment == 0) 
	&& ( input->ccStatReal || input->sigma || input->ccStatComplex) )
   {
     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     /* Write the search summary to the database */
     
     dbEntry = outputPtr->results 
       = (dataBase *) LALMalloc( sizeof(dataBase) );
     
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     
     dbEntry->previous = NULL;

     /* Shared object name */

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "shared_object", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
       
     /* This prevents extra \0s from being written to the strings */
       
     stringLength = strlen(STOCHASTICAPPLYSEARCHH_DSONAME);
     
     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      STOCHASTICAPPLYSEARCHH_DSONAME, stringLength*sizeof(CHAR) );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;
     
     /* LAL CVS tag */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "lal_cvs_tag", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
       
     /* This prevents extra \0s from being written to the strings */
       
     stringLength = strlen(LAL_CVS_TAG);

     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      LAL_CVS_TAG, stringLength*sizeof(CHAR) );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;
     
     /* LALWrapper CVS tag */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "lalwrapper_cvs_tag", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
       
     /* This prevents extra \0s from being written to the strings */
       
     stringLength = strlen(LAL_WRAPPER_CVS_TAG);
     
     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      LAL_WRAPPER_CVS_TAG, stringLength*sizeof(CHAR) );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;
     
     prev = dbEntry;
     
     /* start time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
     strncpy( dbEntry->columnName, "in_start_time", dbNameLimit );
       
     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->timeInterval.startSec;

     prev = dbEntry;
     
     /* start time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_start_time_ns", dbNameLimit );
     
     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->timeInterval.startNan;
       
     prev = dbEntry;
       
     /* end time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_end_time", dbNameLimit );
     
     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->timeInterval.stopSec;
       
     prev = dbEntry;
       
     /* end time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_end_time_ns", dbNameLimit );
     
     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->timeInterval.stopNan;

     prev = dbEntry;
       
     /* start time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
     strncpy( dbEntry->columnName, "out_start_time", dbNameLimit );
       
     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->timeInterval.startSec;
     
     prev = dbEntry;
     
     /* start time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_start_time_ns", dbNameLimit );
     
     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->timeInterval.startNan;
       
     prev = dbEntry;
       
     /* end time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_end_time", dbNameLimit );
     
     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->timeInterval.stopSec;
       
     prev = dbEntry;
       
     /* end time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_end_time_ns", dbNameLimit );
     
     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->timeInterval.stopNan;
     
     dbEntry->next = NULL;
     
     ++outputPtr;

     /* Now write the length of a segment associated with this search job */

     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     /* Write the length of a segment to the database */
     
     dbEntry = outputPtr->results 
       = (dataBase *) LALMalloc( sizeof(dataBase) );
     
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     
     dbEntry->previous = NULL;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "name", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
     
     /* This prevents extra \0s from being written to the strings */
     
     stringLength = strlen(STOCHASTICAPPLYSEARCHH_SEGLENNAME);

     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      STOCHASTICAPPLYSEARCHH_SEGLENNAME,
	      stringLength*sizeof(CHAR) );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;
     
     prev = dbEntry;

     /* value */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "value", dbNameLimit );
     
     dbEntry->type       = real_4;
     dbEntry->numberRows = 1;
     dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
     if ( ! dbEntry->rows.real4 )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;

     *(dbEntry->rows.real4) = params->segmentDuration;

     prev = dbEntry;

     /* Put units in the string field */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "string", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
     
     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	  status );
     TRY( LALUnitAsString(status->statusPtr, unitString, &lalSecondUnit),
	  status );
     
     /* This prevents extra \0s from being written to the strings */
     
     stringLength = strlen(unitString->data);
     
     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      unitString->data, stringLength*sizeof(CHAR) );

     TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	  status );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
     *(dbEntry->rowDimensions) = stringLength;

     dbEntry->next = NULL;
     
     ++outputPtr;
   }

   if (input->ccSpec)
   {
     /* Find end time of segment */
     segStartNanoSeconds = 1000000000L 
       * (INT8) input->ccSpec->epoch.gpsSeconds;
     segStartNanoSeconds += (INT8) input->ccSpec->epoch.gpsNanoSeconds;
     segLengthNanoSeconds = (INT8) 
       (1.0e9 * (UINT8) (params->initParams->numPoints)
	* params->timeInterval.timeStepSize);
     segStopNanoSeconds = segStartNanoSeconds + segLengthNanoSeconds;
     stopSec = (INT4) (segStopNanoSeconds / 1000000000L);
     stopNan = (INT4) (segStopNanoSeconds % 1000000000L);

     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     if ( STOCHASTICH_CCSPECDBBIT & input->outputType )
     {
       
       /* Write the CC spectrum to the database */
       
       dbEntry = outputPtr->results 
	 = (dataBase *) LALMalloc( sizeof(dataBase) );
       
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       
       dbEntry->previous = NULL;
       
       /* start time seconds */
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit);
       strncpy( dbEntry->columnName, "start_time", dbNameLimit );
       
       dbEntry->type       = int_4s;
       dbEntry->numberRows = 1;
       dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
       if ( ! dbEntry->rows.int4s )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->rowDimensions = NULL;
       *(dbEntry->rows.int4s) = (INT4) input->ccSpec->epoch.gpsSeconds;
       
       prev = dbEntry;
       
       /* start time nanoseconds */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "start_time_ns", dbNameLimit );
       
       dbEntry->type       = int_4s;
       dbEntry->numberRows = 1;
       dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
       if ( ! dbEntry->rows.int4s )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->rowDimensions = NULL;
       *(dbEntry->rows.int4s) = (INT4) input->ccSpec->epoch.gpsNanoSeconds;
       
       prev = dbEntry;
       
       /* end time seconds */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "end_time", dbNameLimit );

       dbEntry->type       = int_4s;
       dbEntry->numberRows = 1;
       dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
       if ( ! dbEntry->rows.int4s )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->rowDimensions = NULL;
       *(dbEntry->rows.int4s) = stopSec;
       
       prev = dbEntry;
       
       /* end time nanoseconds */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "end_time_ns", dbNameLimit );
       
       dbEntry->type       = int_4s;
       dbEntry->numberRows = 1;
       dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
       if ( ! dbEntry->rows.int4s )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->rowDimensions = NULL;
       *(dbEntry->rows.int4s) = stopNan;
       
       prev = dbEntry;
       
       /* frames used (?) */
       /* :NOTE: we are assuming all frames consist of whole GPS seconds */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "frames_used", dbNameLimit );
       
       dbEntry->type       = int_4s;
       dbEntry->numberRows = 1;
       dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
       if ( ! dbEntry->rows.int4s )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->rowDimensions = NULL;
       
       *(dbEntry->rows.int4s)
	 = stopSec - (INT4) input->ccSpec->epoch.gpsSeconds;
       
       if ( stopNan != 0 ) *(dbEntry->rows.int4s) += 1;
       
       prev = dbEntry;
       
       /* start frequency */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "start_frequency", dbNameLimit );
       
       dbEntry->type       = real_8;
       dbEntry->numberRows = 1;
       dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
       if ( ! dbEntry->rows.real8 )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->rowDimensions = NULL;
       *(dbEntry->rows.real8) = input->ccSpec->f0;
       
       prev = dbEntry;
       
       /* frequency spacing */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "delta_frequency", dbNameLimit );
       
       dbEntry->type       = real_8;
       dbEntry->numberRows = 1;
       dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
       if ( ! dbEntry->rows.real8 )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->rowDimensions = NULL;
       *(dbEntry->rows.real8) = input->ccSpec->deltaF;
       
       prev = dbEntry;
       
       /* Key for program name. */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "program", dbNameLimit );
       
       dbEntry->type       = char_s_ptr;
       dbEntry->numberRows = 1;
       
       /* This prevents extra \0s from being written to the strings */
       
       stringLength = strlen(STOCHASTICAPPLYSEARCHH_PROGNAME);
       
       dbEntry->rows.chars = 
	 (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
       if ( ! dbEntry->rows.chars )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       strncpy( dbEntry->rows.chars, 
		STOCHASTICAPPLYSEARCHH_PROGNAME, stringLength*sizeof(CHAR) );
       
       dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
       if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       *(dbEntry->rowDimensions) = stringLength;
       
       prev = dbEntry;
       
       /* Key for spectrum type. */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "mimetype", dbNameLimit );
       
       dbEntry->type       = char_s_ptr;
       dbEntry->numberRows = 1;
       
       stringLength = strlen(STOCHASTICAPPLYSEARCHH_MIMETYPE);
       
       dbEntry->rows.chars = 
	 (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
       if ( ! dbEntry->rows.chars )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       strncpy( dbEntry->rows.chars, 
		STOCHASTICAPPLYSEARCHH_MIMETYPE, stringLength*sizeof(CHAR) );
       
       dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
       if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       *(dbEntry->rowDimensions) = stringLength;
       
       prev = dbEntry;
       
       /* key for channel name */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "channel", dbNameLimit );
       
       dbEntry->type       = char_s_ptr;
       dbEntry->numberRows = 1;

       /* TODO: Is this right??? */
       
       stringLength = strlen(STOCHASTICAPPLYSEARCHH_CHANNEL);
       
       dbEntry->rows.chars = 
	 (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
       if ( ! dbEntry->rows.chars )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       strncpy( dbEntry->rows.chars, STOCHASTICAPPLYSEARCHH_CHANNEL
		, stringLength*sizeof(CHAR) );
       
       dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
       if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       *(dbEntry->rowDimensions) = stringLength;
       
       prev = dbEntry;
       
       /* Key for spectrum type. */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "spectrum_type", dbNameLimit );
       
       dbEntry->type       = char_s_ptr;
       dbEntry->numberRows = 1;
       
       stringLength = strlen(STOCHASTICAPPLYSEARCHH_SPECTYPE);
       
       dbEntry->rows.chars = 
	 (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
       if ( ! dbEntry->rows.chars )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       strncpy( dbEntry->rows.chars, 
		STOCHASTICAPPLYSEARCHH_SPECTYPE, stringLength*sizeof(CHAR) );
       
       dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
       if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       *(dbEntry->rowDimensions) = stringLength;
       
       prev = dbEntry;
       
       /* spectrum */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;

       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "spectrum", dbNameLimit );
       
       /* Spectrum must be stored as a BLOb */
       
       dbEntry->type       = char_u_ptr;
       dbEntry->numberRows = 1;
       dbEntry->rowDimensions 
	 = (UINT4 *) LALMalloc( dbEntry->numberRows * sizeof(UINT4));
       dbEntry->rowDimensions[0]
	 = input->ccSpec->data->length * (sizeof(COMPLEX8) / sizeof(UCHAR)) ;
       dbEntry->rows.charu 
	 = (UCHAR *) LALMalloc (dbEntry->rowDimensions[0] * sizeof(UCHAR));
       if ( !(dbEntry->rows.charu) ) 
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       
       /* This is ugly, but should work if there's no padding in storage
	* of COMPLEX8*  */

       memcpy( dbEntry->rows.charu, input->ccSpec->data->data, 
	       dbEntry->rowDimensions[0] * sizeof(UCHAR) );
       
       prev = dbEntry;

       /* spectrum_length */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
       strncpy( dbEntry->columnName, "spectrum_length", dbNameLimit );
       
       dbEntry->type       = int_4s;
       dbEntry->numberRows = 1;
       dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
       if ( ! dbEntry->rows.int4s )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       *(dbEntry->rows.int4s) = input->ccSpec->data->length;
       dbEntry->rowDimensions = NULL;
       
       dbEntry->next = NULL;
     }

     if ( STOCHASTICH_CCSPECFRBIT & input->outputType )
     {
       /* 
	*
	* alloc memory for the optional linked list
	*
	*/
       
       
       /* allocate memory */
       /* using LALCalloc protects us against new members of the structure */
       mddEntry = outputPtr->optional = (multiDimData *) 
	 LALCalloc (1, sizeof (multiDimData));
       if (!mddEntry)
       {
	 ABORT (status, 
		STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC);
       }
       
       /* write the name and domain of the data */
       strncpy (mddEntry->name, "filter_output", maxMultiDimName);
       
       /* Read the dimensions from the output frequency series */
       
       unitString = NULL;
       TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	    status );
       TRY( LALUnitAsString(status->statusPtr, unitString, &(input->ccSpec->sampleUnits)),
	    status );
       strncpy (mddEntry->units, unitString->data, maxMultiDimUnits);
       TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	    status );
       mddEntry->space = freqD;
       
       mddEntry->range.dFreq.numberSamples = input->ccSpec->data->length;
       
       
       mddEntry->range.dFreq.gpsStartTimeSec 
	 = input->ccSpec->epoch.gpsSeconds;
       mddEntry->range.dFreq.gpsStartTimeNan 
	 = input->ccSpec->epoch.gpsNanoSeconds;
       mddEntry->range.dFreq.gpsStopTimeSec = stopSec;
       mddEntry->range.dFreq.gpsStopTimeNan = stopNan;
       
       mddEntry->range.dFreq.startFreq = input->ccSpec->f0;
       mddEntry->range.dFreq.stopFreq 
	 = input->ccSpec->f0 + (REAL8)(input->ccSpec->data->length - 1) * input->ccSpec->deltaF;
       mddEntry->range.dFreq.freqStepSize = input->ccSpec->deltaF;
       /* Set the heterodyning base frequency to -1 because it's meaningless */
       mddEntry->range.dFreq.baseFreq = -1;
       
       
       /* data type is COMPLEX8 */
       mddEntry->type = complex_8;
       
       /* size of the data */
       mddEntry->numberDimensions = 1;
       mddEntry->dimensions = (UINT4 *) LALMalloc (sizeof(UINT4));
       if (!(mddEntry->dimensions))
       {
	 ABORT (status, 
		STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       mddEntry->dimensions[0] = input->ccSpec->data->length;
       
       /* dcHistory is meaningless here */
       mddEntry->history = NULL;
       
       /* allocate memory for the data */
       mddEntry->data.complex8 = (COMPLEX8 *) 
	 LALCalloc (input->ccSpec->data->length, sizeof(COMPLEX8));
       if (!(mddEntry->data.complex8))
       {
	 ABORT (status, 
		STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
   
       /* and copy the data there */
       memcpy (mddEntry->data.complex8, 
	       input->ccSpec->data->data, 
	       input->ccSpec->data->length * sizeof(COMPLEX8) );
     }
     ++outputPtr;
   }
   if ( input->sigma )
   {
     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     /* Write sigma to the database */
     
     dbEntry = outputPtr->results 
       = (dataBase *) LALMalloc( sizeof(dataBase) );
     
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     
     dbEntry->previous = NULL;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "name", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
     
     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen(STOCHASTICAPPLYSEARCHH_SIGMANAME);
     
     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      STOCHASTICAPPLYSEARCHH_SIGMANAME,
	      stringLength*sizeof(CHAR) );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;
     
     prev = dbEntry;

     /* value */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "value", dbNameLimit );
     
     dbEntry->type       = real_4;
     dbEntry->numberRows = 1;
     dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
     if ( ! dbEntry->rows.real4 )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.real4) = input->sigma->value;

     prev = dbEntry;

     /* Put units in the string field */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "string", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
     
     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	  status );
     TRY( LALUnitAsString(status->statusPtr, unitString, &(input->sigma->units)),
	  status );
     
     /* This prevents extra \0s from being written to the strings */
     
     stringLength = strlen(unitString->data);
     
     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      unitString->data, stringLength*sizeof(CHAR) );

     TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	  status );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
     *(dbEntry->rowDimensions) = stringLength;

     dbEntry->next = NULL;
     
     ++outputPtr;
   }
   if ( input->ccStatReal )
   {
     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     /* Write the CC statistic to the database */
     
     dbEntry = outputPtr->results 
       = (dataBase *) LALMalloc( sizeof(dataBase) );
     
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     
     dbEntry->previous = NULL;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "name", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
     
     /* This prevents extra \0s from being written to the strings */
     
     sprintf(string, STOCHASTICAPPLYSEARCHH_CCSTATREALNAME,
	     params->currentSegment);

     stringLength = strlen(string);
     
     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      string,
	      stringLength*sizeof(CHAR) );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;
     
     prev = dbEntry;

     /* value */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "value", dbNameLimit );
     
     dbEntry->type       = real_4;
     dbEntry->numberRows = 1;
     dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
     if ( ! dbEntry->rows.real4 )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.real4) = input->ccStatReal->value;

     prev = dbEntry;

     /* Put units in the string field */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     dbEntry->previous = prev;
     
     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "string", dbNameLimit );
     
     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;
     
     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	  status );
     TRY( LALUnitAsString(status->statusPtr, unitString, &(input->ccStatReal->units)),
	  status );
     
     /* This prevents extra \0s from being written to the strings */
     
     stringLength = strlen(unitString->data);
     
     dbEntry->rows.chars = 
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	      STOCHASTICAPPLYSEARCHH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, 
	      unitString->data, stringLength*sizeof(CHAR) );

     TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	  status );
     
     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
     *(dbEntry->rowDimensions) = stringLength;

     dbEntry->next = NULL;
     
     ++outputPtr;
   }
   else if ( input->ccStatComplex )
   {
     for ( i=0; i<2; ++i ) {
       /* this is the same for every output */
       outputPtr->indexNumber = 0;
       outputPtr->search         = stocastic;
       /* :TODO: note the incorrect spelling above.  it was entered */
       /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
       /* that there is another misspelling in that same file:       */
       /* catagory should spelled category */
       outputPtr->significant    = STOCHASTICH_TRUE;
       outputPtr->states         = NULL;
       outputPtr->optional       = NULL;
       
       /* Write the CC statistic to the database */
       
       dbEntry = outputPtr->results 
	 = (dataBase *) LALMalloc( sizeof(dataBase) );
       
       if ( ! dbEntry )
       {
	   ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		  STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       
       dbEntry->previous = NULL;
       
       strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
       strncpy( dbEntry->columnName, "name", dbNameLimit );
       
       dbEntry->type       = char_s_ptr;
       dbEntry->numberRows = 1;
       
       /* This prevents extra \0s from being written to the strings */
       
       sprintf(string, ( i ? STOCHASTICAPPLYSEARCHH_CCSTATIMAGPARTNAME 
			 : STOCHASTICAPPLYSEARCHH_CCSTATREALPARTNAME ),
	       params->currentSegment);

       stringLength = strlen(string);
       
       dbEntry->rows.chars = 
	 (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
       if ( ! dbEntry->rows.chars )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       strncpy( dbEntry->rows.chars, 
		string,
		stringLength*sizeof(CHAR) );
       
       dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
       if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       *(dbEntry->rowDimensions) = stringLength;
     
       prev = dbEntry;

       /* value */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
       strncpy( dbEntry->columnName, "value", dbNameLimit );
       
       dbEntry->type       = real_4;
       dbEntry->numberRows = 1;
       dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
       if ( ! dbEntry->rows.real4 )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->rowDimensions = NULL;
       *(dbEntry->rows.real4) = ( i ? 
				  input->ccStatComplex->value.im :
				  input->ccStatComplex->value.re );
       prev = dbEntry;

       /* Put units in the string field */
       dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
       if ( ! dbEntry )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       dbEntry->previous = prev;
       
       strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
       strncpy( dbEntry->columnName, "string", dbNameLimit );
       
       dbEntry->type       = char_s_ptr;
       dbEntry->numberRows = 1;
       
       unitString = NULL;
       TRY( LALCHARCreateVector(status->statusPtr, &unitString, 
				LALUnitTextSize),
	    status );
       TRY( LALUnitAsString(status->statusPtr, unitString, 
			    &(input->ccStatComplex->units)),
	    status );
     
       /* This prevents extra \0s from being written to the strings */
     
       stringLength = strlen(unitString->data);
       
       dbEntry->rows.chars = 
	 (CHAR *) LALMalloc( stringLength * dbEntry->numberRows 
			     * sizeof(CHAR) );
       if ( ! dbEntry->rows.chars )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       strncpy( dbEntry->rows.chars, 
		unitString->data, stringLength*sizeof(CHAR) );
       
       TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	    status );
       
       dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
       if ( ! dbEntry->rowDimensions )
       {
	 ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       *(dbEntry->rowDimensions) = stringLength;
       
       dbEntry->next = NULL;
       
       ++outputPtr;
     }
   }

   /* normal exit */
   DETATCHSTATUSPTR (status);
   RETURN (status);
}
