/***************************** <lalVerbatim file="StochasticInitSearchCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticInitSearch.c,v 1.30 2002/06/14 19:07:08 whelan Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticInitSearchC">
\subsection{Module \texttt{StochasticInitSearch.c}}
\label{s:StochasticInitSearch.c}

Routine that checks the validity of the input parameters, initializes 
them, and then allocates memory for certain data structures used in the 
stochastic search.

\subsubsection*{Prototypes}
\input{StochasticInitSearchCP}
\index{\texttt{LALInitSearch()}}

\subsection*{Description}

Checks the validity of the input parameters, initializes them, and 
then allocates memory for the time-series input data, optimal filter,
and windowing functions.

\subsection*{Algorithms}
  
\subsection*{Uses}

\begin{itemize}
\item \texttt{LALMalloc}
\item \texttt{LALCCreateVector}
\item \texttt{LALCreateForwardRealFFTPlan}
\item \texttt{LALCreateStochasticDataSegmentVector}
\end{itemize}

\subsection*{Notes}

\vfill{\footnotesize\input{StochasticInitSearchCV}}

*************************************************** </lalLaTeX> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/TimeFreqFFT.h>
#include <lal/DetectorSite.h>
#include <lal/Window.h>
#include "StochasticData.h"
#include "StochasticInitSearch.h"

NRCSID (STOCHASTICINITSEARCHC, "$Id: StochasticInitSearch.c,v 1.30 2002/06/14 19:07:08 whelan Exp $");

/************************ <lalVerbatim file="StochasticInitSearchCP"> */
void LALInitSearch(
    LALStatus*              status,
    void**                  searchParams,
    LALInitSearchParams*    initSearchParams
)
/*********************************************** </lalVerbatim> */
{
    StochasticSearchParams *params = NULL;
    
    REAL8 fMin, fMinOut;

    INITSTATUS (status, "LALInitSearch", STOCHASTICINITSEARCHC);
    ATTATCHSTATUSPTR (status);

#ifdef STOCHASTIC_SO_DEBUG
    lalDebugLevel = LALALLDBG;
#endif

    /* sleep to give me time to attach ddd */
#ifdef STOCHASTIC_SO_ATTACH_DDD
    {
        unsigned int doze = 10;
        pid_t myPID;
        myPID = getpid();
        fprintf (stdout, "pid %d sleeping for %d seconds\n", myPID, doze );
        fflush (stdout);
        sleep (doze);
        fprintf (stdout, "pid %d awake\n", myPID);
        fflush (stdout);
    }
#endif

    /* Check architecture to make sure BLOb we write is sensible */

    if ( sizeof(UCHAR) != 1 || sizeof(COMPLEX8) != 8 )
    {
      ABORT( status, STOCHASTICINITSEARCHH_EDTSZ, 
	     STOCHASTICINITSEARCHH_MSGEDTSZ );
    }

    /*
     *  
     * check input, output, and params
     *  
     */
    
    ASSERT (searchParams, 
            status,
            STOCHASTICINITSEARCHH_ENULL, 
            STOCHASTICINITSEARCHH_MSGENULL );

    ASSERT (!(*searchParams), 
            status,
            STOCHASTICINITSEARCHH_ENNUL, 
            STOCHASTICINITSEARCHH_MSGENNUL );

    ASSERT (initSearchParams, 
            status,
            STOCHASTICINITSEARCHH_ENULL, 
            STOCHASTICINITSEARCHH_MSGENULL );

    /* check command line arguments */
 
    if (!(initSearchParams->argv)) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENULL, 
               STOCHASTICINITSEARCHH_MSGENULL);
    }

    if (initSearchParams->argc != STOCHASTICFILTERPARAM_NPARAMS) 
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EARGS, 
               STOCHASTICINITSEARCHH_MSGEARGS ); 
    }
    if (strcmp (initSearchParams->argv[STOCHASTICFILTERPARAM_FILTERPARAMS],
		"-filterparams" )) 
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EARG0, 
               STOCHASTICINITSEARCHH_MSGEARG0);
    }
    if (atoi (initSearchParams->argv[STOCHASTICFILTERPARAM_NUMSEGMENTS]) <= 0) 
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENSEG, 
               STOCHASTICINITSEARCHH_MSGENSEG);
    }
    if (atoi (initSearchParams->argv[STOCHASTICFILTERPARAM_NUMPOINTS]) <= 0)
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENPTS, 
               STOCHASTICINITSEARCHH_MSGENPTS);
    }
    if (atof (initSearchParams->argv[STOCHASTICFILTERPARAM_BASEFREQ]) < 0.0)
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EBASE, 
               STOCHASTICINITSEARCHH_MSGEBASE);
    }
    if ((UINT4) 
	atoi( initSearchParams->argv[STOCHASTICFILTERPARAM_WINDOWTYPE1] )
	>= NumberWindowTypes )
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EUWIN, 
               STOCHASTICINITSEARCHH_MSGEUWIN);
    }
    if ((UINT4) 
	atoi( initSearchParams->argv[STOCHASTICFILTERPARAM_WINDOWTOP1] )
	> atoi (initSearchParams->argv[STOCHASTICFILTERPARAM_NUMPOINTS] ))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENWIN, 
               STOCHASTICINITSEARCHH_MSGENWIN);
    }
    if ((UINT4) 
	atoi( initSearchParams->argv[STOCHASTICFILTERPARAM_WINDOWTYPE2] )
	>= NumberWindowTypes )
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EUWIN, 
               STOCHASTICINITSEARCHH_MSGEUWIN);
    }
    if ((UINT4) 
	atoi( initSearchParams->argv[STOCHASTICFILTERPARAM_WINDOWTOP2] )
	> atoi (initSearchParams->argv[STOCHASTICFILTERPARAM_NUMPOINTS] ))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENWIN, 
               STOCHASTICINITSEARCHH_MSGENWIN);
    }
    /* :TODO: check windowing parameters */

    if (atoi( initSearchParams->argv[STOCHASTICFILTERPARAM_NUMFREQUENCIES] )
	< 1 )
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENFRQ, 
               STOCHASTICINITSEARCHH_MSGENFRQ);
    }
    if (atof (initSearchParams->argv[STOCHASTICFILTERPARAM_F0]) < 0)
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EF0, 
               STOCHASTICINITSEARCHH_MSGEF0);
    }
    if (atof (initSearchParams->argv[STOCHASTICFILTERPARAM_DELTAF]) <= 0) 
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EDELF, 
               STOCHASTICINITSEARCHH_MSGEDELF);
    }
    if (atoi ( initSearchParams->argv[STOCHASTICFILTERPARAM_NUMFREQUENCIES] ) < 1 )
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENFRQ, 
               STOCHASTICINITSEARCHH_MSGENFRQ);
    }
    if (atof (initSearchParams->argv[STOCHASTICFILTERPARAM_F0]) < 0) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EF0, 
               STOCHASTICINITSEARCHH_MSGEF0);
    }
    if (atof (initSearchParams->argv[STOCHASTICFILTERPARAM_DELTAF]) <= 0) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EDELF, 
               STOCHASTICINITSEARCHH_MSGEDELF);
    }
    
    /* 
     *
     * allocate memory for parameter structure 
     *
     */
    
    *searchParams = params = 
        LALMalloc (sizeof (StochasticSearchParams)); 
    if (!params)
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }

    params->initParams = (StochasticInitParams *)
        LALMalloc (sizeof (StochasticInitParams));
    if (!(params->initParams))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }

        
    /*
     *
     *  assign the initial parameter values 
     *
     */

    params->initParams->numSegments     
      = (UINT4)
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_NUMSEGMENTS]);
    params->initParams->numPoints       
      = (UINT4)
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_NUMPOINTS]); 
    params->initParams->baseFreq
      = atof (initSearchParams->argv[STOCHASTICFILTERPARAM_BASEFREQ]);
    params->initParams->windowType1
      = (WindowType) 
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_WINDOWTYPE1]);
    params->initParams->windowTop1
      = (UINT4) 
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_WINDOWTOP1]);
    params->initParams->windowType2
      = (WindowType) 
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_WINDOWTYPE2]);
    params->initParams->windowTop2
      = (UINT4) 
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_WINDOWTOP2]);
    params->initParams->fullLength
      = (UINT4) 
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_FULLLENGTH]);
    params->initParams->numFrequencies 
      = (UINT4) 
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_NUMFREQUENCIES]);
    params->initParams->f0     
      = atof (initSearchParams->argv[STOCHASTICFILTERPARAM_F0]);
    params->initParams->deltaF 
      = atof (initSearchParams->argv[STOCHASTICFILTERPARAM_DELTAF]);
    params->initParams->outputNumFreq
      = (UINT4) 
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_NUMFREQUENCIES]);
    params->initParams->outputF0    
      = atof (initSearchParams->argv[STOCHASTICFILTERPARAM_F0]);
    params->initParams->outputDeltaF 
      = atof (initSearchParams->argv[STOCHASTICFILTERPARAM_DELTAF]);

    params->initParams->baseFreq
      = atof (initSearchParams->argv[STOCHASTICFILTERPARAM_BASEFREQ]);

    params->initParams->epochsMatch
      = (BOOLEAN)
      atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_EPOCHSMATCH]);
    params->initParams->siteID1
      = *(initSearchParams->argv[STOCHASTICFILTERPARAM_SITEID1]);
    params->initParams->siteID2
      = *(initSearchParams->argv[STOCHASTICFILTERPARAM_SITEID2]);

/* AZIMUTH CHANGES */
    if (strlen(initSearchParams->argv[STOCHASTICFILTERPARAM_SITEID1]) > 1)
    {
      params->initParams->azimuth1 = (REAL4*) LALMalloc(sizeof(REAL4));
      if (!(params->initParams->azimuth1))
      {
	ABORT (status, 
	       STOCHASTICINITSEARCHH_EALOC, 
	       STOCHASTICINITSEARCHH_MSGEALOC);
      }
      *(params->initParams->azimuth1) 
	= atof(initSearchParams->argv[STOCHASTICFILTERPARAM_SITEID1]+1);
    }
    else 
    {
      params->initParams->azimuth1 = NULL;
    }
    if (strlen(initSearchParams->argv[STOCHASTICFILTERPARAM_SITEID2]) > 1)
    {
      params->initParams->azimuth2 = (REAL4*) LALMalloc(sizeof(REAL4));
      if (!(params->initParams->azimuth2))
      {
	ABORT (status, 
	       STOCHASTICINITSEARCHH_EALOC, 
	       STOCHASTICINITSEARCHH_MSGEALOC);
      }
      *(params->initParams->azimuth2) 
	= atof(initSearchParams->argv[STOCHASTICFILTERPARAM_SITEID2]+1);
    }
    else 
    {
      params->initParams->azimuth2 = NULL;
    }
    params->initParams->outputType 
      = (UINT4) atoi(initSearchParams->argv[STOCHASTICFILTERPARAM_OUTPUTTYPE]);

    /* Check consistency of arguments */

    if (params->initParams->outputDeltaF < params->initParams->deltaF)
    {
      ABORT (status, 
	     STOCHASTICINITSEARCHH_EFRES, 
	     STOCHASTICINITSEARCHH_MSGEFRES);      
    }

    if (params->initParams->numFrequencies > 
          ((2*params->initParams->numPoints)-1))
    {
      ABORT (status, 
	     STOCHASTICINITSEARCHH_EFNUM, 
	     STOCHASTICINITSEARCHH_MSGEFNUM);      
    }

    fMin = params->initParams->f0 
      || params->initParams->f0 - ( params->initParams->deltaF / 2 );
    fMinOut = params->initParams->outputF0 
      || params->initParams->outputF0 
      - ( params->initParams->outputDeltaF / 2 );

    if (fMinOut < fMin)
    {
      ABORT (status, 
	     STOCHASTICINITSEARCHH_EFRNG, 
	     STOCHASTICINITSEARCHH_MSGEFRNG);      
    }

    if ( params->initParams->outputF0 
	   + ( (REAL8) params->initParams->outputNumFreq - 0.5 ) 
	     * params->initParams->outputDeltaF 
	 > params->initParams->f0
	   + ( (REAL8) params->initParams->numFrequencies - 0.5 ) 
	     * params->initParams->deltaF )
    {
      ABORT (status, 
	     STOCHASTICINITSEARCHH_EFRNG, 
	     STOCHASTICINITSEARCHH_MSGEFRNG);      
    }

    /* :TODO: check that searchMaster and haveData are set properly */
    /* I think these are right! WGA */
    params->searchMaster    = initSearchParams->nodeClass; 
    params->haveData        = 0;
    params->currentSegment  = 0;
    
    
    /* 
     *
     * allocate memory for non-trivial parameter fields 
     *
     */
   
    /* window functions */
    
    params->window1 = NULL;
    LALSCreateVector (status->statusPtr,
		      &(params->window1),
		      params->initParams->numPoints);
    CHECKSTATUSPTR (status); 
    
    params->window2 = NULL;
    LALSCreateVector (status->statusPtr,
		      &(params->window2),
		      params->initParams->numPoints);
    CHECKSTATUSPTR (status);

    if ( ( STOCHASTICH_CCSTATDBBIT | STOCHASTICH_CCSPECDBBIT 
	   | STOCHASTICH_CCSPECFRBIT )
	 & params->initParams->outputType )
    {
      /* optimal filter */

      params->optimalFilter = (COMPLEX8FrequencySeries *) 
        LALMalloc (sizeof (COMPLEX8FrequencySeries));

      if (!(params->optimalFilter))
      {
	ABORT (status, 
	       STOCHASTICINITSEARCHH_EALOC, 
	       STOCHASTICINITSEARCHH_MSGEALOC);
      }
    
      params->optimalFilter->data = NULL;
      LALCCreateVector (status->statusPtr,
			&(params->optimalFilter->data),
			params->initParams->numFrequencies);
      CHECKSTATUSPTR (status);
      
      /* fftPlan MUST CHANGE BASED ON BASE FREQ */
      
      if (!(params->initParams->baseFreq))
      {
	params->fftPlan.realPlan = NULL;
	LALCreateForwardRealFFTPlan(status->statusPtr,
				    &(params->fftPlan.realPlan),
				    params->initParams->fullLength,
				    STOCHASTICH_FALSE);
	CHECKSTATUSPTR (status);
	
	/* data segment vector MUST CHANGE BASED ON BASE FREQ */     
	params->stochasticSegVec.realVec = NULL;
	LALSCreateStochasticDataSegmentVector
	  (status->statusPtr, 
	   &(params->stochasticSegVec.realVec), 
	   params->initParams);
	CHECKSTATUSPTR (status);
      }
      else
      {
	params->fftPlan.complexPlan = NULL;
	LALCreateForwardComplexFFTPlan(status->statusPtr,
				       &(params->fftPlan.complexPlan),
				       params->initParams->fullLength,
				       STOCHASTICH_FALSE);
	CHECKSTATUSPTR (status);
	
	/* data segment vector MUST CHANGE BASED ON BASE FREQ */
	params->stochasticSegVec.complexVec = NULL;
	LALCCreateStochasticDataSegmentVector 
	  (status->statusPtr,
	   &(params->stochasticSegVec.complexVec),
	   params->initParams);
	CHECKSTATUSPTR (status);
      }
    }
    else
    {
      params->optimalFilter = NULL;
    }

    if ( STOCHASTICH_CCSIGMDBBIT & params->initParams->outputType )
    {
      params->sigma = (REAL4WithUnits *) LALMalloc (sizeof (REAL4WithUnits));
      if (!(params->sigma))
      {
	ABORT (status, 
	       STOCHASTICINITSEARCHH_EALOC, 
	       STOCHASTICINITSEARCHH_MSGEALOC);
      }
    }
    else 
    {
      params->sigma = NULL;
    }
    
    /* normal exit */
    DETATCHSTATUSPTR (status);
    RETURN (status);
}

